/*
Script: MuseScore_Hotkeys_v1.1.ahk 
Author: "Wyvern". 
Date: 3rd May 2017.

This Script is designed to enhance the Rhythm Entry mode available in MuseScore 2.1.
It allows the left-hand to select Note/Rest/Dot/Tuplet modes with the Control keys,
while the right-hand simultaneously enters a duration using the Number Pad. It is a
less sophisticated version of the Denemo approach.

The Script requires AutoHotkeys Version 1.1.24.05 or later. The software is Open 
Source, and may be downloaded from the AutoHotkey Foundation under License GNU GPLv2. 
SciTE4AutoHotkey is highly recommended for editing and debugging the Script.
A compiled *.exe version can also be created that runs without AutoHotkey installed.

The Directive "#IfWinActive ahk_exe MuseScore.exe" ensures that the Hotkeys are only 
enabled while the MuseScore Window is Active.

You are welcome to use, modify, or otherwise adapt the Script as you see fit. 
However, this is entirely at your own risk. Please use the MuseScore forum to provide
"Wyvern" with helpful feedback and suggestions for improving the Script.

The Number Pad 'Modifiers' are shown below.

NumPad (1-9) : Note
Win+NumPad (1-9) : Rest
Alt+NumPad (1-9) : Dotted Note
Win+Alt+NumPad (1-9) : Dotted Rest
Ctrl+NumPad (2-9) : Tuplet (acts on the previous duration entered)

Ctrl+Alt+H : Help Box
Ctrl+Alt+S : Suspend/Resume Hotkeys (allows normal MuseScore action)
Ctrl+Alt+X : Exit

; -------------------------------------------------------------------------------------
; Scratch Pad 
; -------------------------------------------------------------------------------------
; To edit the Script, comment-out the "#IfWinActive..." lines. It is highly recommended 
; that you use a Scratch Pad to avoid overwriting your code with 'Send' characters!
; Just place your mouse cursor in the Scratch Pad area before triggering any Hotkeys.
;
;
; 
;
;
;
;
;
*/

; -------------------------------------------------------------------------------------
; Directives
; -------------------------------------------------------------------------------------

#NoEnv                  ; Recommended for compatibility with new AutoHotkey releases.
#SingleInstance Force   ; Replaces script (Reloads).
#Warn                   ; Enables warnings to assist with detecting common errors.

; -------------------------------------------------------------------------------------
; Initialisation
; -------------------------------------------------------------------------------------

SendMode, Event                 ; Allows a delay to be set between keystrokes.
SetKeyDelay, 10                 ; The (default) delay in milliseconds.
SetWorkingDir %A_ScriptDir%     ; Ensures a consistent starting directory.      
SetNumLockState, On             ; In case we forget!
Suspend, On                     ; Suspends Hotkeys until after the initial Help Box.

; -------------------------------------------------------------------------------------
; Global Variables
; -------------------------------------------------------------------------------------

Buffer := 0         ; The current key character.

; -------------------------------------------------------------------------------------
; Main Script
; -------------------------------------------------------------------------------------

if A_IsSuspended    ; This is true at start-up.
{
   gosub, Help      ; Displays the initial Help Box.
   Suspend, Off     ; Enables the Hotkeys.
}
return 

; -------------------------------------------------------------------------------------
; Always active Hotkeys
; -------------------------------------------------------------------------------------

; Help Hotkey
^!h::
   Suspend, Permit
   gosub, Help
   return

; Suspend/Resume Hotkey
^!s::
   Suspend
   if A_IsSuspended 
      MsgBox, 0, MuseScore HotKeys. Version 1.1.
      , Hotkeys are suspended.`n`nCtrl+Alt+S to Resume.
      , 5   ; Five second timeout.
   else
      MsgBox, 0, MuseScore HotKeys. Version 1.1.
      , Hotkeys are active!`n`nCtrl+Alt+S to Suspend.
      , 5   ; Five second timeout.
   return
   
; Exit Application Hotkey
^!x::
   Suspend
   ExitApp
   
;-------------------------------------------------------------------------------------
 
#IfWinActive ahk_exe MuseScore.exe ; Enables Hotkeys when MuseScore Window is Active.   

;-------------------------------------------------------------------------------------
; Control Keys (Modifiers)
; -------------------------------------------------------------------------------------

; Disables the Ctrl keys acting alone.
~LCtrl Up::return
~RCtrl Up::return

; Disables the Win keys acting alone.
~LWin Up::return
~RWin Up::return

; Disables the Alt key acting alone.
~Alt Up::return

; -------------------------------------------------------------------------------------
; Note Hotkeys
; -------------------------------------------------------------------------------------

; Number Pad : No Modifier key.
;Blocked.
Numpad0::
NumpadDot::
   return
; Active 
Numpad1::
Numpad2::
Numpad3::
Numpad4::
Numpad5::
Numpad6::
Numpad7::
Numpad8::
Numpad9::
   ; Retrieves the key value.
   StringTrimLeft, Buffer, A_ThisHotkey, 6
   ; Sends the MuseScore string
   Send, %Buffer%
return

; -------------------------------------------------------------------------------------
; Rest Hotkeys
; -------------------------------------------------------------------------------------

; Number Pad : With Win key (#) pressed.
; Blocked.
#Numpad0::
#NumpadDot::
   return
; Active 
#Numpad1::
#Numpad2::
#Numpad3::
#Numpad4::
#Numpad5::
#Numpad6::
#Numpad7::
#Numpad8::
#Numpad9::
   ; Retrieves the key value.
   StringTrimLeft, Buffer, A_ThisHotkey, 7
   ; Sends the MuseScore string
   Send, 0%Buffer%0 ; '0' is the MS Rest key.
return

; -------------------------------------------------------------------------------------
; Dotted Note Hotkeys
; -------------------------------------------------------------------------------------

; Number Pad : With Alt key (!) pressed.
; Blocked.
!Numpad0::
!NumpadDot::
   return
; Active 
!Numpad1::
!Numpad2::
!Numpad3::
!Numpad4::
!Numpad5::
!Numpad6::
!Numpad7::
!Numpad8::
!Numpad9::
   ; Retrieves the key value.
   StringTrimLeft, Buffer, A_ThisHotkey, 7
   ; Sends the MuseScore string
   Send, .%Buffer%. ; '.' is the MS Dot key   
return

; -------------------------------------------------------------------------------------
; Dotted Rest Hotkeys
; -------------------------------------------------------------------------------------

; Number Pad : With Win and Alt keys (#!) pressed.
; Blocked.
#!Numpad0::
#!NumpadDot::
   return
; Active   
#!Numpad1::
#!Numpad2::
#!Numpad3::
#!Numpad4::
#!Numpad5::
#!Numpad6::
#!Numpad7::
#!Numpad8::
#!Numpad9::
   ; Retrieves the key value.
   StringTrimLeft, Buffer, A_ThisHotkey, 8
   ; Sends the MuseScore string
Send, 0.%Buffer%0.  ; '0' and '.' are the MS Rest and Dot characters, respectively.   
return

; -------------------------------------------------------------------------------------
; Tuplet Hotkeys
; -------------------------------------------------------------------------------------

; Number Pad : With Ctrl key (^) pressed.
; Blocked.
^Numpad0::
^Numpad1::
^NumpadDot::
   return
; Active   
^Numpad2::
^Numpad3::
^Numpad4::
^Numpad5::
^Numpad6::
^Numpad7::
^Numpad8::
^Numpad9::
   ; Retrieves the key value.
   StringTrimLeft, Buffer, A_ThisHotkey, 7
   ; Sends the MuseScore string, shifting left to use the previous key duration.
   Send, {Left}^%Buffer% ; Cursor left, Ctrl+Tuplet.   
return

#IfWinActive    ; Turns off the initial Directive. Just to be tidy!

; -------------------------------------------------------------------------------------
; Subroutines
; -------------------------------------------------------------------------------------

; Help Box:  Press Ctrl+Alt+H to display.
Help:
MsgBox, 0, MuseScore HotKeys. Version 1.1., 
(
This AutoHotkey Script should be used with the
MuseScore Rhythm Entry mode. The Number Pad
enters Note or Rest durations. The Win and Alt
keys modify the Number Pad, as specified in
the list below. They are pressed 'on the fly' 
while entering the durations (Tuplets excepted).
The Tuplets act on the prior duration entered.

NumPad (1-9) : Note
Win+NumPad (1-9) : Rest
Alt+NumPad (1-9) : Dotted Note
Win+Alt+NumPad (1-9) : Dotted Rest
Ctrl+NumPad (2-9) : Tuplet

Ctrl+Alt+H : This Help Box
Ctrl+Alt+S : Suspend/Resume Hotkeys
Ctrl+Alt+X : Exit
)
IfWinNotActive ahk_exe nightly.exe
   MsgBox, 0, MuseScore HotKeys. Version 1.1.
   , The MuseScore Window is not Active!
   , 5   ; Five second timeout.   
return
